#!/usr/bin/env python

# flake8: noqa

from __future__ import absolute_import
from nose.tools import assert_equal

import linchpin.InventoryFilters
from linchpin.InventoryFilters import InventoryProviders

from linchpin.InventoryFilters.AWSInventory import AWSInventory
from linchpin.InventoryFilters.BeakerInventory import BeakerInventory
from linchpin.InventoryFilters.DuffyInventory import DuffyInventory
from linchpin.InventoryFilters.DummyInventory import DummyInventory
from linchpin.InventoryFilters.GCloudInventory import GCloudInventory
from linchpin.InventoryFilters.LibvirtInventory import LibvirtInventory
from linchpin.InventoryFilters.OpenstackInventory import OpenstackInventory
from linchpin.InventoryFilters.OvirtInventory import OvirtInventory
from linchpin.InventoryFilters.VMwareInventory import VMwareInventory
from linchpin.InventoryFilters.DockerInventory import DockerInventory
from linchpin.InventoryFilters.AzureInventory import AzureInventory
from linchpin.InventoryFilters.OpenshiftInventory import OpenshiftInventory
def test_get_driver():
    driver = InventoryProviders.get_driver("aws_ec2_res")
    assert_equal(driver, AWSInventory)

def test_get_all_drivers():
    drivers = {
        "azure_vm" : AzureInventory,
        "aws_ec2_res": AWSInventory,
        "beaker_res": BeakerInventory,
        "duffy_res": DuffyInventory,
        "dummy_res": DummyInventory,
        "nummy_res": DummyInventory,
        "gcloud_gce_res": GCloudInventory,
        "libvirt_res": LibvirtInventory,
        "os_server_res": OpenstackInventory,
        "ovirt_vms_res": OvirtInventory,
        "vmware_guest_res": VMwareInventory,
        "docker_container_res": DockerInventory,
        "docker_image_res": DockerInventory,
        "openshift_res": OpenshiftInventory,
    }
    if_drivers = InventoryProviders.get_all_drivers()
    assert_equal(drivers,if_drivers)



"""
Inventory Providers List
"""
from __future__ import absolute_import
from linchpin.exceptions import LinchpinError

from .AWSInventory import AWSInventory
from .BeakerInventory import BeakerInventory
from .DuffyInventory import DuffyInventory
from .DummyInventory import DummyInventory
from .GCloudInventory import GCloudInventory
from .LibvirtInventory import LibvirtInventory
from .OpenstackInventory import OpenstackInventory
from .OvirtInventory import OvirtInventory
from .VMwareInventory import VMwareInventory
from .DockerInventory import DockerInventory
from .AzureInventory import AzureInventory
from .OpenshiftInventory import OpenshiftInventory

from .JSONInventoryFormatter import JSONInventoryFormatter
from .CFGInventoryFormatter import CFGInventoryFormatter

filter_classes = {
    "azure_vm": AzureInventory,
    "aws_ec2_res": AWSInventory,
    "beaker_res": BeakerInventory,
    "duffy_res": DuffyInventory,
    "dummy_res": DummyInventory,
    "nummy_res": DummyInventory,
    "gcloud_gce_res": GCloudInventory,
    "libvirt_res": LibvirtInventory,
    "os_server_res": OpenstackInventory,
    "ovirt_vms_res": OvirtInventory,
    "vmware_guest_res": VMwareInventory,
    "docker_container_res": DockerInventory,
    "docker_image_res": DockerInventory,
    "openshift_res": OpenshiftInventory
}

formatter_classes = {
    "cfg": CFGInventoryFormatter,
    "ini": CFGInventoryFormatter,
    "json": JSONInventoryFormatter
}


def get_driver(provider):

    if provider not in filter_classes:
        raise LinchpinError("Key {0} not found in"
                            " inventory provider dict".format(provider))

    return filter_classes[provider]


def get_all_drivers():
    return filter_classes


def get_inv_formatter(inv_type):
    if inv_type not in formatter_classes:
        raise LinchpinError("Key {0} not found in"
                            " formatters".format(inv_type))
    return formatter_classes[inv_type]


def get_all_inv_formatters():
    return formatter_classes



from __future__ import print_function
import os
import json
import sys

workspace = os.listdir(".")

print("output to stderr: {0}".format(json.dumps(workspace)))
print(json.dumps(workspace), file=sys.stderr)




#!/usr/bin/env python
import linchpin.FilterUtils.FilterUtils as filter_utils


class FilterModule(object):
    ''' A filter to add_res_type '''
    def filters(self):
        return {
            'add_res_type': filter_utils.add_res_type
        }



from __future__ import absolute_import
from collections import OrderedDict

from .InventoryFilter import InventoryFilter


class AzureInventory(InventoryFilter):
    # DEFAULT_HOSTNAMES = ['public_dns_name', 'public_ip', 'private_ip']

    def get_host_data(self, res, cfgs):
        """
        Returns a dict of hostnames or IP addresses for use in an Ansible
        inventory file, based on available data. Only a single hostname or IP
        address will be returned per instance, so as to avoid duplicate runs of
        Ansible on the same host via the generated inventory file.

        Each hostname contains mappings of any variable that was defined in the
        cfgs section of the PinFile (e.g. __IP__) to the value in the field that
        corresponds with that variable in the cfgs.

        :param topo:
            linchpin Azure VM resource data

        :param cfgs:
            map of config options from PinFile
        """

        host_data = OrderedDict()
        if res['resource_type'] != 'azure_vm':
            return host_data
        networks = res['properties']['networkProfile']['networkInterfaces']
        for network in networks:
            mid = network['properties']['ipConfigurations'][0]
            # mid: this is a middle variable for flake test
            ip = mid['properties']['publicIPAddress']['properties']['ipAddress']
            host_data[ip] = {"__IP__": ip}
        return host_data



from __future__ import print_function
import sys
import json


prev_hooks = sys.argv[-1]
print("input: {0}".format(prev_hooks))

prev_hooks = json.loads(prev_hooks)


# return successfully if the PinFile is in the list of files
for hook in prev_hooks:
    if type(hook['data']) is list and 'PinFile' in hook['data']:
        print('found PinFile')
        sys.exit(0)

error = {'error': 'could not find \'PinFile\'', 'files': files}
print(error, file=sys.stderr)
sys.exit(1)



from __future__ import absolute_import
from collections import OrderedDict
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
from .InventoryFilter import InventoryFilter


class DockerInventory(InventoryFilter):
    DEFAULT_HOSTNAMES = ['Config.Hostname']

    def get_host_data(self, res, config):
        host_data = OrderedDict()
        # Only docker_container resource type produces host data.
        if res['resource_type'] != 'docker_container_res':
            return host_data
        var_data = config.get('docker', {})
        if var_data is None:
            var_data = {}
        host = self.get_hostname(res, var_data,
                                 self.DEFAULT_HOSTNAMES)
        hostname_var = host[0]
        hostname = host[1]
        host_data[hostname] = {}
        if '__IP__' not in var_data.keys():
            var_data['__IP__'] = hostname_var
        self.set_config_values(host_data[hostname], res, var_data)
        return host_data

    def get_inventory(self, res, layout, config):
        output = StringIO()
        self.config.write(output)

        return output.getvalue()



#!/usr/bin/env python
from __future__ import absolute_import
from collections import OrderedDict

from .InventoryFilter import InventoryFilter


class OvirtInventory(InventoryFilter):
    # we should find a better set of defaults at some point
    DEFAULT_HOSTNAMES = ['ips.address_v4']


    def get_host_data(self, res, cfgs):
        """
        Returns a dict of hostnames or IP addresses for use in an Ansible
        inventory file, based on available data. Only a single hostname or IP
        address will be returned per instance, so as to avoid duplicate runs of
        Ansible on the same host via the generated inventory file.

        Each hostname contains mappings of any variable that was defined in the
        cfgs section of the PinFile (e.g. __IP__) to the value in the field that
        corresponds with that variable in the cfgs.

        By default, the hostname will be the system field returned by ovirt

        :param topo:
            linchpin oVirt resource data

        :param cfgs:
            map of config options from PinFile
        """

        host_data = OrderedDict()
        if res['resource_type'] != 'ovirt_vms_res':
            return host_data
        var_data = cfgs.get('ovirt', {})
        if var_data is None:
            var_data = {}
        if res['vm']['reported_devices']:
            for dev in res['vm']['reported_devices']:
                host = self.get_hostname(dev, var_data,
                                         self.DEFAULT_HOSTNAMES)
                hostname_var = host[0]
                hostname = host[1]
                host_data[hostname] = {}
                if '__IP__' not in list(var_data.keys()):
                    var_data['__IP__'] = hostname_var
                    host_data[hostname] = {}
                self.set_config_values(host_data[hostname], dev, var_data)
        return host_data


    def config_value_helper(self, instance, keys):
        if "." in keys:
            key, rest = keys.split('.', 1)
            # this handles errors in which the key does not exist
            if isinstance(instance, list) and key.isdigit():
                return self.config_value_helper(instance[int(key)], rest)
            if key not in list(instance.keys()):
                return ''
            return self.config_value_helper(instance[key], rest)
        else:
            if keys == '':
                return ''
            elif keys == 'address_v4':
                for addr in instance:
                    if addr['version'] == 'v4':
                        return addr['address']
            elif keys == 'address_v6':
                for addr in instance:
                    if addr['version'] == 'v6':
                        return addr['address']
            elif keys not in list(instance.keys()):
                return ''
            return instance[keys]


    def get_hostname(self, data, cfgs, default_fields):
        if '__IP__' in list(cfgs.keys()):
            val = self.config_value_helper(data, cfgs['__IP__'])
            if val:
                return (cfgs['__IP__'], val)
        for var in default_fields:
            val = self.config_value_helper(data, var)
            if val:
                return (var, val)
        return ''
